@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-bookmarks.c 8324 2008-06-30 18:22:09Z chpe $
+ *  $Id$
  */
 
 #include "config.h"
@@ -44,8 +44,7 @@
 
 #include <string.h>
 #include <glib/gi18n.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkdialog.h>
+#include <gtk/gtk.h>
 
 #ifdef ENABLE_ZEROCONF
 #include <avahi-common/error.h>
@@ -961,7 +960,10 @@ resolver_found_cb (GaServiceResolver *resolver,
 	
 	ephy_node_db_set_immutable (priv->db, was_immutable);
 
-	g_hash_table_unref (text_table);
+	if (text_table != NULL)
+	{
+		g_hash_table_unref (text_table);
+	}
 }
 
 static void
