@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2003, 2004 Marco Pesenti Gritti
  *  Copyright © 2003, 2004 Christian Persch
@@ -16,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-navigation-action.c 7135 2007-07-10 12:52:35Z chpe $
+ *  $Id$
  */
 
 #include "config.h"
@@ -26,23 +27,17 @@
 #include "ephy-window.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-history.h"
+#include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
+#include "ephy-history-item.h"
 #include "ephy-link.h"
 #include "ephy-gui.h"
 #include "ephy-debug.h"
 
-#include <gtk/gtklabel.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtkmenushell.h>
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkmenutoolbutton.h>
-#include <gtk/gtktoolbar.h>
-#include <gtk/gtkstatusbar.h>
+#include <gtk/gtk.h>
 
-#define NTH_DATA_KEY	"GoNTh"
-#define URL_DATA_KEY	"GoURL"
+#define HISTORY_ITEM_DATA_KEY	"HistoryItem"
+#define URL_DATA_KEY	        "GoURL"
 
 #define EPHY_NAVIGATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NAVIGATION_ACTION, EphyNavigationActionPrivate))
 
@@ -65,35 +60,7 @@ enum
 static void ephy_navigation_action_init       (EphyNavigationAction *action);
 static void ephy_navigation_action_class_init (EphyNavigationActionClass *class);
 
-static GObjectClass *parent_class = NULL;
-
-GType
-ephy_navigation_action_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo type_info =
-		{
-			sizeof (EphyNavigationActionClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) ephy_navigation_action_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (EphyNavigationAction),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) ephy_navigation_action_init,
-		};
-
-		type = g_type_register_static (EPHY_TYPE_LINK_ACTION,
-					       "EphyNavigationAction",
-					       &type_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EphyNavigationAction, ephy_navigation_action, EPHY_TYPE_LINK_ACTION)
 
 #define MAX_LABEL_LENGTH 48
 
@@ -138,40 +105,26 @@ new_history_menu_item (const char *origtext,
 }
 
 static void
-activate_by_history_index (EphyNavigationAction *action,
-			   int index)
+activate_back_or_forward_menu_item_cb (GtkWidget *menuitem,
+				       EphyNavigationAction *action)
 {
+	EphyHistoryItem *item;
 	EphyEmbed *embed;
 
-	embed = ephy_window_get_active_embed (action->priv->window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (action->priv->window));
 	g_return_if_fail (embed != NULL);
 
 	if (ephy_gui_is_middle_click ())
 	{
-		EphyEmbed *dest;
-		EphyTab *newTab;
-
-		newTab = ephy_link_open (EPHY_LINK (action), "about:blank", NULL,
-					 EPHY_LINK_NEW_TAB);
-		g_return_if_fail (newTab != NULL);
-
-		dest = ephy_tab_get_embed (newTab);
-		g_return_if_fail (dest != NULL);
-
-		embed = dest;
+		embed = ephy_link_open (EPHY_LINK (action), "about:blank", NULL,
+				        EPHY_LINK_NEW_TAB);
+		g_return_if_fail (embed != NULL);
 	}
-	ephy_embed_shistory_go_nth (embed, index);
-}
 
-static void
-activate_back_or_forward_menu_item_cb (GtkWidget *menuitem,
-				       EphyNavigationAction *action)
-{
-	int go_nth;
+	item = (EphyHistoryItem*)g_object_get_data (G_OBJECT (menuitem), HISTORY_ITEM_DATA_KEY);
+	g_return_if_fail (item != NULL);
 
-	go_nth = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menuitem), NTH_DATA_KEY));
-
-	activate_by_history_index(action, go_nth);
+	ephy_embed_go_to_history_item (embed, item);
 }
 
 static void
@@ -180,25 +133,34 @@ select_menu_item_cb (GtkWidget *menuitem,
 {
 	char *url;
 	GtkWidget *statusbar;
+	EphyHistoryItem *item;
+	char *freeme = NULL;
 
-	url = g_object_get_data (G_OBJECT (menuitem), URL_DATA_KEY);
-	g_return_if_fail (url != NULL);
+	item = (EphyHistoryItem*)g_object_get_data (G_OBJECT (menuitem), HISTORY_ITEM_DATA_KEY);
+	if (item)
+	{
+		url = ephy_history_item_get_url (item);
+		freeme = url;
+	}
+	else
+	{
+		url = g_object_get_data (G_OBJECT (menuitem), URL_DATA_KEY);
+		g_return_if_fail (url != NULL);
+	}
 
 	statusbar = ephy_window_get_statusbar (action->priv->window);
 
 	gtk_statusbar_push (GTK_STATUSBAR (statusbar), action->priv->statusbar_cid, url);
+
+	g_free (freeme);
 }
 
 static void
 deselect_menu_item_cb (GtkWidget *menuitem,
 	               EphyNavigationAction *action)
 {
-	char *url;
 	GtkWidget *statusbar;
 
-	url = g_object_get_data (G_OBJECT (menuitem), URL_DATA_KEY);
-	g_return_if_fail (url != NULL);
-
 	statusbar = ephy_window_get_statusbar (action->priv->window);
 
 	gtk_statusbar_pop (GTK_STATUSBAR (statusbar), action->priv->statusbar_cid);
@@ -211,7 +173,7 @@ activate_up_menu_item_cb (GtkWidget *menuitem,
 	EphyEmbed *embed;
 	char *url;
 
-	embed = ephy_window_get_active_embed (action->priv->window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (action->priv->window));
 	g_return_if_fail (embed != NULL);
 
 	url = g_object_get_data (G_OBJECT (menuitem), URL_DATA_KEY);
@@ -227,45 +189,36 @@ build_back_or_forward_menu (EphyNavigationAction *action)
 	EphyWindow *window = action->priv->window;
 	GtkMenuShell *menu;
 	EphyEmbed *embed;
-	int pos, count;
-	int start, end;
+	GList *list, *l;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_val_if_fail (embed != NULL, NULL);
 
-	pos = ephy_embed_shistory_get_pos (embed);
-	count = ephy_embed_shistory_n_items (embed);
-
-	if (count == 0) return NULL;
-
 	if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_BACK)
-	{
-		start = pos - 1;
-		end = -1;
-	}
+		list = ephy_embed_get_backward_history (embed);
 	else
-	{
-		start = pos + 1;
-		end = count;
-	}
+		list = ephy_embed_get_forward_history (embed);
 
 	menu = GTK_MENU_SHELL (gtk_menu_new ());
 
-	while (start != end)
+	l = list;
+
+	for (l = list; l != NULL; l = l->next)
 	{
 		GtkWidget *item;
-		char *title = NULL, *url = NULL;
+		EphyHistoryItem *hitem;
+		char *title, *url;
 
-		ephy_embed_shistory_get_nth (embed, start, FALSE, &url, &title);
-
-		if (url == NULL) continue;
+		hitem = (EphyHistoryItem*)l->data;
+		url = ephy_history_item_get_url (hitem);
+		title = ephy_history_item_get_title (hitem);
 
 		item = new_history_menu_item (title ? title : url, url);
 
-		g_object_set_data (G_OBJECT (item), NTH_DATA_KEY,
-				   GINT_TO_POINTER (start));
-		g_object_set_data_full (G_OBJECT (item), URL_DATA_KEY, url, 
-					(GDestroyNotify) g_free);
+		g_free (title);
+
+		g_object_set_data_full (G_OBJECT (item), HISTORY_ITEM_DATA_KEY, hitem,
+					(GDestroyNotify) g_object_unref);
 
 		g_signal_connect (item, "activate",
 				  G_CALLBACK (activate_back_or_forward_menu_item_cb),
@@ -279,18 +232,9 @@ build_back_or_forward_menu (EphyNavigationAction *action)
 
 		gtk_menu_shell_append (menu, item);
 		gtk_widget_show_all (item);
-
-		g_free (title);
-
-		if (start < end)
-		{
-			start++;
-		}
-		else
-		{
-			start--;
-		}
 	}
+	
+	g_list_free (list);
 
 	return GTK_WIDGET (menu);
 }
@@ -300,25 +244,38 @@ build_up_menu (EphyNavigationAction *action)
 {
 	EphyWindow *window = action->priv->window;
 	EphyEmbed *embed;
+	EphyHistory *history;
 	GtkMenuShell *menu;
 	GtkWidget *item;
 	GSList *list, *l;
 	char *url;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_val_if_fail (embed != NULL, NULL);
 
 	menu = GTK_MENU_SHELL (gtk_menu_new ());
 
+    	history = EPHY_HISTORY
+		(ephy_embed_shell_get_global_history (embed_shell));
+
 	list = ephy_embed_get_go_up_list (embed);
 
 	for (l = list; l != NULL; l = l->next)
 	{
+		EphyNode *node;
+		const char *title = NULL;
+
 		url = l->data;
 
 		if (url == NULL) continue;
 
-		item = new_history_menu_item (url, url);
+		node = ephy_history_get_page (history, url);
+		if (node != NULL)
+		{
+			title = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_TITLE);
+		}
+
+		item = new_history_menu_item (title ? title : url, url);
 
 		g_object_set_data_full (G_OBJECT (item), URL_DATA_KEY, url,
 					(GDestroyNotify) g_free);
@@ -386,7 +343,7 @@ connect_proxy (GtkAction *gaction,
 				  G_CALLBACK (menu_activated_cb), gaction);
 	}
 
-	GTK_ACTION_CLASS (parent_class)->connect_proxy (gaction, proxy);
+	GTK_ACTION_CLASS (ephy_navigation_action_parent_class)->connect_proxy (gaction, proxy);
 }
 
 static void
@@ -395,27 +352,42 @@ ephy_navigation_action_activate (GtkAction *gtk_action)
 	EphyNavigationAction *action = EPHY_NAVIGATION_ACTION (gtk_action);
 	EphyWindow *window = action->priv->window;
 	EphyEmbed *embed;
-	int pos;
 
-	embed = ephy_window_get_active_embed (window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	pos = ephy_embed_shistory_get_pos (embed);
-
 	if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_BACK)
 	{
-		activate_by_history_index (action, pos - 1);
+		if (ephy_gui_is_middle_click ())
+		{
+			embed = ephy_link_open (EPHY_LINK (action),
+						"about:blank",
+						NULL,
+						EPHY_LINK_NEW_TAB);
+		}
+		ephy_embed_go_back (embed);
 	}
 	else if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_FORWARD)
 	{
-		activate_by_history_index (action, pos + 1);
+		if (ephy_gui_is_middle_click ())
+		{
+			embed = ephy_link_open (EPHY_LINK (action),
+						"about:blank",
+						NULL,
+						EPHY_LINK_NEW_TAB);
+		}
+		ephy_embed_go_forward (embed);
 	}
 	else if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_UP)
 	{
+		GSList *up_list;
+		
+		up_list = ephy_embed_get_go_up_list (embed);
 		ephy_link_open (EPHY_LINK (action),
-				ephy_embed_get_go_up_list (embed)->data,
+				up_list->data,
 				NULL,
 				ephy_gui_is_middle_click () ? EPHY_LINK_NEW_TAB : 0);
+		g_slist_free (up_list);
 	}
 }
 
@@ -432,7 +404,7 @@ ephy_navigation_action_finalize (GObject *object)
 
 	g_free (action->priv->arrow_tooltip);
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (ephy_navigation_action_parent_class)->finalize (object);
 }
 
 static void
@@ -501,8 +473,6 @@ ephy_navigation_action_class_init (EphyNavigationActionClass *class)
 	object_class->set_property = ephy_navigation_action_set_property;
 	object_class->get_property = ephy_navigation_action_get_property;
 
-	parent_class = g_type_class_peek_parent (class);
-
 	action_class->toolbar_item_type = GTK_TYPE_MENU_TOOL_BUTTON;
 	action_class->connect_proxy = connect_proxy;
 	action_class->activate = ephy_navigation_action_activate;
